/*
 * ManufacturerController.java
 *
 */

package simpleHibernateApp.controller;

import java.util.ArrayList;
import java.util.Collection;
import javax.annotation.Resource;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceUnit;
import javax.persistence.Query;
import javax.transaction.UserTransaction;
import simpleHibernateApp.agents.simpleHibernateAppAgent;
import simpleHibernateApp.entities.Manufacturer;

/**
 *
 * @author maxpoon@dev.java.net
 */
public class ManufacturerController {
    
    /** Creates a new instance of ManufacturerController */
    public ManufacturerController() { }

    private Manufacturer manufacturer;

    private DataModel model;

    @Resource
    private UserTransaction utx;

    @PersistenceUnit(unitName = "SimpleHibernateAppPU")
    private EntityManagerFactory emf;

    private EntityManager getEntityManager() {
    	  initJMX();
        return emf.createEntityManager();
    }

    // Initiate JMX if needed
    void initJMX() {
        try {
            simpleHibernateAppAgent.getDefault();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    private int batchSize = 20;

    private int firstItem = 0;

    public Manufacturer getManufacturer() {
        return manufacturer;
    }

    public void setManufacturer(Manufacturer manufacturer) {
        this.manufacturer = manufacturer;
    }

    public DataModel getDetailManufacturers() {
        return model;
    }

    public void setDetailManufacturers(Collection<Manufacturer> m) {
        model = new ListDataModel(new ArrayList(m));
    }

    public String createSetup() {
        this.manufacturer = new Manufacturer();
        return "manufacturer_create";
    }

    public String create() {
        EntityManager em = null;
        try {
            utx.begin();
            em = getEntityManager();
            em.persist(manufacturer);
            utx.commit();
            addSuccessMessage("Manufacturer was successfully created.");
        } catch (Exception ex) {
            try {
                addErrorMessage(ex.getLocalizedMessage());
                utx.rollback();
            } catch (Exception e) {
                addErrorMessage(e.getLocalizedMessage());
            }
        } finally {
            em.close();
        }
        return "manufacturer_list";
    }

    public String detailSetup() {
        setManufacturerFromRequestParam();
        return "manufacturer_detail";
    }

    public String editSetup() {
        setManufacturerFromRequestParam();
        return "manufacturer_edit";
    }

    public String edit() {
        EntityManager em = null;
        try {
            utx.begin();
            em = getEntityManager();
            manufacturer = em.merge(manufacturer);
            utx.commit();
            addSuccessMessage("Manufacturer was successfully updated.");
        } catch (Exception ex) {
            try {
                addErrorMessage(ex.getLocalizedMessage());
                utx.rollback();
            } catch (Exception e) {
                addErrorMessage(e.getLocalizedMessage());
            }
        } finally {
            em.close();
        }
        return "manufacturer_list";
    }

    public String destroy() {
        EntityManager em = null;
        try {
            utx.begin();
            Manufacturer manufacturer = getManufacturerFromRequestParam();
            em = getEntityManager();
            manufacturer = em.merge(manufacturer);
            em.remove(manufacturer);
            utx.commit();
            addSuccessMessage("Manufacturer was successfully deleted.");
        } catch (Exception ex) {
            try {
                addErrorMessage(ex.getLocalizedMessage());
                utx.rollback();
            } catch (Exception e) {
                addErrorMessage(e.getLocalizedMessage());
            }
        } finally {
            em.close();
        }
        return "manufacturer_list";
    }

    public Manufacturer getManufacturerFromRequestParam() {
        EntityManager em = getEntityManager();
        try{
            Manufacturer o = (Manufacturer) model.getRowData();
            o = em.merge(o);
            return o;
        } finally {
            em.close();
        }
    }

    public void setManufacturerFromRequestParam() {
        Manufacturer manufacturer = getManufacturerFromRequestParam();
        setManufacturer(manufacturer);
    }

    public DataModel getManufacturers() {
        EntityManager em = getEntityManager();
        try{
            Query q = em.createQuery("select object(o) from Manufacturer as o");
            q.setMaxResults(batchSize);
            q.setFirstResult(firstItem);
            model = new ListDataModel(q.getResultList());
            return model;
        } finally {
            em.close();
        }
    }

    public static void addErrorMessage(String msg) {
        FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msg);
        FacesContext fc = FacesContext.getCurrentInstance();
        fc.addMessage(null, facesMsg);
    }

    public static void addSuccessMessage(String msg) {
        FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO, msg, msg);
        FacesContext fc = FacesContext.getCurrentInstance();
        fc.addMessage("successInfo", facesMsg);
    }

    public Manufacturer findManufacturer(Integer id) {
        EntityManager em = getEntityManager();
        try{
            Manufacturer o = (Manufacturer) em.find(Manufacturer.class, id);
            return o;
        } finally {
            em.close();
        }
    }

    public int getItemCount() {
        EntityManager em = getEntityManager();
        try{
            int count = ((Long) em.createQuery("select count(o) from Manufacturer as o").getSingleResult()).intValue();
            return count;
        } finally {
            em.close();
        }
    }

    public int getFirstItem() {
        return firstItem;
    }

    public int getLastItem() {
        int size = getItemCount();
        return firstItem + batchSize > size ? size : firstItem + batchSize;
    }

    public int getBatchSize() {
        return batchSize;
    }

    public String next() {
        if (firstItem + batchSize < getItemCount()) {
            firstItem += batchSize;
        }
        return "manufacturer_list";
    }

    public String prev() {
        firstItem -= batchSize;
        if (firstItem < 0) {
            firstItem = 0;
        }
        return "manufacturer_list";
    }
    
}
