/*
 * ProductCodeController.java
 *
 */

package simpleHibernateApp.controller;

import java.util.ArrayList;
import java.util.Collection;
import javax.annotation.Resource;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceUnit;
import javax.persistence.Query;
import javax.transaction.UserTransaction;
import simpleHibernateApp.agents.simpleHibernateAppAgent;
import simpleHibernateApp.entities.ProductCode;

/**
 *
 * @author maxpoon@dev.java.net
 */
public class ProductCodeController {
    
    /** Creates a new instance of ProductCodeController */
    public ProductCodeController() { }

    private ProductCode productCode;

    private DataModel model;

    @Resource
    private UserTransaction utx;

    @PersistenceUnit(unitName = "SimpleHibernateAppPU")
    private EntityManagerFactory emf;

    private EntityManager getEntityManager() {
    	  initJMX();
        return emf.createEntityManager();
    }

    // Initiate JMX if needed
    void initJMX() {
        try {
            simpleHibernateAppAgent.getDefault();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    private int batchSize = 20;

    private int firstItem = 0;

    public ProductCode getProductCode() {
        return productCode;
    }

    public void setProductCode(ProductCode productCode) {
        this.productCode = productCode;
    }

    public DataModel getDetailProductCodes() {
        return model;
    }

    public void setDetailProductCodes(Collection<ProductCode> m) {
        model = new ListDataModel(new ArrayList(m));
    }

    public String createSetup() {
        this.productCode = new ProductCode();
        return "productCode_create";
    }

    public String create() {
        EntityManager em = null;
        try {
            utx.begin();
            em = getEntityManager();
            em.persist(productCode);
            utx.commit();
            addSuccessMessage("ProductCode was successfully created.");
        } catch (Exception ex) {
            try {
                addErrorMessage(ex.getLocalizedMessage());
                utx.rollback();
            } catch (Exception e) {
                addErrorMessage(e.getLocalizedMessage());
            }
        } finally {
            em.close();
        }
        return "productCode_list";
    }

    public String detailSetup() {
        setProductCodeFromRequestParam();
        return "productCode_detail";
    }

    public String editSetup() {
        setProductCodeFromRequestParam();
        return "productCode_edit";
    }

    public String edit() {
        EntityManager em = null;
        try {
            utx.begin();
            em = getEntityManager();
            productCode = em.merge(productCode);
            utx.commit();
            addSuccessMessage("ProductCode was successfully updated.");
        } catch (Exception ex) {
            try {
                addErrorMessage(ex.getLocalizedMessage());
                utx.rollback();
            } catch (Exception e) {
                addErrorMessage(e.getLocalizedMessage());
            }
        } finally {
            em.close();
        }
        return "productCode_list";
    }

    public String destroy() {
        EntityManager em = null;
        try {
            utx.begin();
            ProductCode productCode = getProductCodeFromRequestParam();
            em = getEntityManager();
            productCode = em.merge(productCode);
            em.remove(productCode);
            utx.commit();
            addSuccessMessage("ProductCode was successfully deleted.");
        } catch (Exception ex) {
            try {
                addErrorMessage(ex.getLocalizedMessage());
                utx.rollback();
            } catch (Exception e) {
                addErrorMessage(e.getLocalizedMessage());
            }
        } finally {
            em.close();
        }
        return "productCode_list";
    }

    public ProductCode getProductCodeFromRequestParam() {
        EntityManager em = getEntityManager();
        try{
            ProductCode o = (ProductCode) model.getRowData();
            o = em.merge(o);
            return o;
        } finally {
            em.close();
        }
    }

    public void setProductCodeFromRequestParam() {
        ProductCode productCode = getProductCodeFromRequestParam();
        setProductCode(productCode);
    }

    public DataModel getProductCodes() {
        EntityManager em = getEntityManager();
        try{
            Query q = em.createQuery("select object(o) from ProductCode as o");
            q.setMaxResults(batchSize);
            q.setFirstResult(firstItem);
            model = new ListDataModel(q.getResultList());
            return model;
        } finally {
            em.close();
        }
    }

    public static void addErrorMessage(String msg) {
        FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msg);
        FacesContext fc = FacesContext.getCurrentInstance();
        fc.addMessage(null, facesMsg);
    }

    public static void addSuccessMessage(String msg) {
        FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO, msg, msg);
        FacesContext fc = FacesContext.getCurrentInstance();
        fc.addMessage("successInfo", facesMsg);
    }

    public ProductCode findProductCode(String id) {
        EntityManager em = getEntityManager();
        try{
            ProductCode o = (ProductCode) em.find(ProductCode.class, id);
            return o;
        } finally {
            em.close();
        }
    }

    public int getItemCount() {
        EntityManager em = getEntityManager();
        try{
            int count = ((Long) em.createQuery("select count(o) from ProductCode as o").getSingleResult()).intValue();
            return count;
        } finally {
            em.close();
        }
    }

    public int getFirstItem() {
        return firstItem;
    }

    public int getLastItem() {
        int size = getItemCount();
        return firstItem + batchSize > size ? size : firstItem + batchSize;
    }

    public int getBatchSize() {
        return batchSize;
    }

    public String next() {
        if (firstItem + batchSize < getItemCount()) {
            firstItem += batchSize;
        }
        return "productCode_list";
    }

    public String prev() {
        firstItem -= batchSize;
        if (firstItem < 0) {
            firstItem = 0;
        }
        return "productCode_list";
    }
    
}
