/*
 * ProductController.java
 *
 */

package simpleHibernateApp.controller;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import javax.annotation.Resource;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;
import javax.faces.model.SelectItem;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceUnit;
import javax.persistence.Query;
import javax.transaction.UserTransaction;
import simpleHibernateApp.agents.simpleHibernateAppAgent;
import simpleHibernateApp.entities.Manufacturer;
import simpleHibernateApp.entities.Product;
import simpleHibernateApp.entities.ProductCode;

/**
 *
 * @author maxpoon@dev.java.net
 */
public class ProductController {
    
    /** Creates a new instance of ProductController */
    public ProductController() { }

    private Product product;

    private DataModel model;

    @Resource
    private UserTransaction utx;

    @PersistenceUnit(unitName = "SimpleHibernateAppPU")
    private EntityManagerFactory emf;

    private EntityManager getEntityManager() {
    	  initJMX();
        return emf.createEntityManager();
    }

    // Initiate JMX if needed
    void initJMX() {
        try {
            simpleHibernateAppAgent.getDefault();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    private int batchSize = 20;

    private int firstItem = 0;

    public Product getProduct() {
        return product;
    }

    public void setProduct(Product product) {
        this.product = product;
    }

    public DataModel getDetailProducts() {
        return model;
    }

    public void setDetailProducts(Collection<Product> m) {
        model = new ListDataModel(new ArrayList(m));
    }

    public String createSetup() {
        this.product = new Product();
        return "product_create";
    }

    public String create() {
        EntityManager em = null;
        try {
            utx.begin();
            em = getEntityManager();
            em.persist(product);
            utx.commit();
            addSuccessMessage("Product was successfully created.");
        } catch (Exception ex) {
            try {
                addErrorMessage(ex.getLocalizedMessage());
                utx.rollback();
            } catch (Exception e) {
                addErrorMessage(e.getLocalizedMessage());
            }
        } finally {
            em.close();
        }
        return "product_list";
    }

    public String detailSetup() {
        setProductFromRequestParam();
        return "product_detail";
    }

    public String editSetup() {
        setProductFromRequestParam();
        return "product_edit";
    }

    public String edit() {
        EntityManager em = null;
        try {
            utx.begin();
            em = getEntityManager();
            product = em.merge(product);
            utx.commit();
            addSuccessMessage("Product was successfully updated.");
        } catch (Exception ex) {
            try {
                addErrorMessage(ex.getLocalizedMessage());
                utx.rollback();
            } catch (Exception e) {
                addErrorMessage(e.getLocalizedMessage());
            }
        } finally {
            em.close();
        }
        return "product_list";
    }

    public String destroy() {
        EntityManager em = null;
        try {
            utx.begin();
            Product product = getProductFromRequestParam();
            em = getEntityManager();
            product = em.merge(product);
            em.remove(product);
            utx.commit();
            addSuccessMessage("Product was successfully deleted.");
        } catch (Exception ex) {
            try {
                addErrorMessage(ex.getLocalizedMessage());
                utx.rollback();
            } catch (Exception e) {
                addErrorMessage(e.getLocalizedMessage());
            }
        } finally {
            em.close();
        }
        return "product_list";
    }

    public Product getProductFromRequestParam() {
        EntityManager em = getEntityManager();
        try{
            Product o = (Product) model.getRowData();
            o = em.merge(o);
            return o;
        } finally {
            em.close();
        }
    }

    public void setProductFromRequestParam() {
        Product product = getProductFromRequestParam();
        setProduct(product);
    }

    public DataModel getProducts() {
        EntityManager em = getEntityManager();
        try{
            Query q = em.createQuery("select object(o) from Product as o");
            q.setMaxResults(batchSize);
            q.setFirstResult(firstItem);
            model = new ListDataModel(q.getResultList());
            return model;
        } finally {
            em.close();
        }
    }

    public static void addErrorMessage(String msg) {
        FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msg);
        FacesContext fc = FacesContext.getCurrentInstance();
        fc.addMessage(null, facesMsg);
    }

    public static void addSuccessMessage(String msg) {
        FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO, msg, msg);
        FacesContext fc = FacesContext.getCurrentInstance();
        fc.addMessage("successInfo", facesMsg);
    }

    public Product findProduct(Integer id) {
        EntityManager em = getEntityManager();
        try{
            Product o = (Product) em.find(Product.class, id);
            return o;
        } finally {
            em.close();
        }
    }

    public javax.faces.model.SelectItem[] getManufacturerIds() {
        EntityManager em = getEntityManager();
        try{
            List <Manufacturer> l = (List <Manufacturer>) em.createQuery("select o from Manufacturer as o").getResultList();
            SelectItem select[] = new SelectItem[l.size()];
            int i = 0;
            for(Manufacturer x : l) {
                    select[i++] = new SelectItem(x);
                }
                return select;
        } finally {
            em.close();
        }
    }

    public javax.faces.model.SelectItem[] getProductCodes() {
        EntityManager em = getEntityManager();
        try{
            List <ProductCode> l = (List <ProductCode>) em.createQuery("select o from ProductCode as o").getResultList();
            SelectItem select[] = new SelectItem[l.size()];
            int i = 0;
            for(ProductCode x : l) {
                    select[i++] = new SelectItem(x);
                }
                return select;
        } finally {
            em.close();
        }
    }

    public int getItemCount() {
        EntityManager em = getEntityManager();
        try{
            int count = ((Long) em.createQuery("select count(o) from Product as o").getSingleResult()).intValue();
            return count;
        } finally {
            em.close();
        }
    }

    public int getFirstItem() {
        return firstItem;
    }

    public int getLastItem() {
        int size = getItemCount();
        return firstItem + batchSize > size ? size : firstItem + batchSize;
    }

    public int getBatchSize() {
        return batchSize;
    }

    public String next() {
        if (firstItem + batchSize < getItemCount()) {
            firstItem += batchSize;
        }
        return "product_list";
    }

    public String prev() {
        firstItem -= batchSize;
        if (firstItem < 0) {
            firstItem = 0;
        }
        return "product_list";
    }
    
}
